@@ -1,4 +1,5 @@
 #define _POSIX_C_SOURCE 199309L
+#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavutil/display.h>
 #include <libavutil/hwcontext_drm.h>
@@ -490,26 +491,27 @@ static void *vid_encode_thread(void *arg) {
 		}
 
 		while (1) {
-			AVPacket pkt;
-			av_init_packet(&pkt);
-
-			int ret = avcodec_receive_packet(ctx->avctx, &pkt);
+			AVPacket *pkt = av_packet_alloc();
+			int ret = avcodec_receive_packet(ctx->avctx, pkt);
 			if (ret == AVERROR(EAGAIN)) {
+				av_packet_free(&pkt);
 				break;
 			} else if (ret == AVERROR_EOF) {
 				av_log(ctx, AV_LOG_INFO, "Encoder flushed!\n");
+				av_packet_free(&pkt);
 				goto end;
 			} else if (ret) {
 				av_log(ctx, AV_LOG_ERROR, "Error encoding: %s!\n",
 						av_err2str(ret));
+				av_packet_free(&pkt);
 				err = ret;
 				goto end;
 			}
 
-			pkt.stream_index = 0;
-			err = av_interleaved_write_frame(ctx->avf, &pkt);
+			pkt->stream_index = 0;
+			err = av_interleaved_write_frame(ctx->avf, pkt);
 
-			av_packet_unref(&pkt);
+			av_packet_free(&pkt);
 
 			if (err) {
 				av_log(ctx, AV_LOG_ERROR, "Writing packet fail: %s!\n",
@@ -618,12 +620,12 @@ static int init_encoding(struct capture_context *ctx) {
 	}
 
 	/* Find encoder */
-	AVCodec *out_codec = avcodec_find_encoder_by_name(ctx->encoder_name);
+	const AVCodec *out_codec = avcodec_find_encoder_by_name(ctx->encoder_name);
 	if (!out_codec) {
 		av_log(ctx, AV_LOG_ERROR, "Codec not found (not compiled in lavc?)!\n");
 		return AVERROR(EINVAL);
 	}
-	ctx->avf->oformat->video_codec = out_codec->id;
+	ctx->avf->oformat = av_guess_format(ctx->encoder_name, NULL, NULL);
 	ctx->is_software_encoder = !(out_codec->capabilities & AV_CODEC_CAP_HARDWARE);
 
 	ctx->avctx = avcodec_alloc_context3(out_codec);
@@ -753,8 +755,11 @@ static int init(struct capture_context *ctx) {
 	ctx->registry = wl_display_get_registry(ctx->display);
 	wl_registry_add_listener(ctx->registry, &registry_listener, ctx);
 
+	// First roundtrip to fetch globals
+	wl_display_roundtrip(ctx->display);
+
+	// Second roundtrip to fetch wl_output information
 	wl_display_roundtrip(ctx->display);
-	wl_display_dispatch(ctx->display);
 
 	if (!ctx->export_manager) {
 		av_log(ctx, AV_LOG_ERROR, "Compositor doesn't support %s!\n",
